<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://ays-pro.com
 * @since      1.0.0
 *
 * @package    Quiz_Maker_Advanced_Display_Quizzes
 * @subpackage Quiz_Maker_Advanced_Display_Quizzes/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Quiz_Maker_Advanced_Display_Quizzes
 * @subpackage Quiz_Maker_Advanced_Display_Quizzes/includes
 * @author     Quiz Maker team <info@ays-pro.com>
 */
class Quiz_Maker_Advanced_Display_Quizzes {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Quiz_Maker_Advanced_Display_Quizzes_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'QUIZ_MAKER_ADVANCED_DISPLAY_QUIZZES_VERSION' ) ) {
			$this->version = QUIZ_MAKER_ADVANCED_DISPLAY_QUIZZES_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'quiz-maker-advanced-display-quizzes';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Quiz_Maker_Advanced_Display_Quizzes_Loader. Orchestrates the hooks of the plugin.
	 * - Quiz_Maker_Advanced_Display_Quizzes_i18n. Defines internationalization functionality.
	 * - Quiz_Maker_Advanced_Display_Quizzes_Admin. Defines all hooks for the admin area.
	 * - Quiz_Maker_Advanced_Display_Quizzes_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-advanced-display-quizzes-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-advanced-display-quizzes-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-quiz-maker-advanced-display-quizzes-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-quiz-maker-advanced-display-quizzes-public.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-advanced-display-table-shortcode.php';
		
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-advanced-display-table-by-category-shortcode.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-advanced-display-list-shortcode.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-advanced-display-list-by-category-shortcode.php';
		
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-advanced-display-with-images.php';

		$this->loader = new Quiz_Maker_Advanced_Display_Quizzes_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Quiz_Maker_Advanced_Display_Quizzes_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Quiz_Maker_Advanced_Display_Quizzes_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Quiz_Maker_Advanced_Display_Quizzes_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// Action hooks | Start

		// Quiz Maker Shorcodes / settings page
		$this->loader->add_action( 'ays_qm_settings_page_extra_shortcodes', $plugin_admin, 'ays_qm_settings_page_advanced_display_quizzes_content' );

		// Action hooks | End

		// ===== Quizzes table shortcode ====
		// Quizzes table shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_advanced_display_quizzes_contents', $plugin_admin, 'ays_quiz_settings_page_adq_table_content', 101, 2 );

		// ===== Quizzes List shortcode ====
		// Quizzes List shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_advanced_display_quizzes_contents', $plugin_admin, 'ays_quiz_settings_page_adq_list_content', 102, 3 );

		// ===== Quizzes Poster shortcode ====
		// Quizzes Poster shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_advanced_display_quizzes_contents', $plugin_admin, 'ays_quiz_settings_page_adq_poster_content', 103, 4 );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Quiz_Maker_Advanced_Display_Quizzes_Public( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_quizzes_table = new Quiz_Maker_Advancesd_Display_Table_shortcode( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_quizzes_table_by_cat = new Quiz_Maker_Advancesd_Display_Table_by_Cat_shortcode( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_quizzes_list = new Quiz_Maker_Advancesd_Display_List_shortcode( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_quizzes_list_by_cat = new Quiz_Maker_Advancesd_Display_List_by_Cat_shortcode( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_quizzes_img_list = new Quiz_Maker_Advancesd_Display_with_Images_shortcode( $this->get_plugin_name(), $this->get_version() );


		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Quiz_Maker_Advanced_Display_Quizzes_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
