<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://ays-pro.com/
 * @since      1.0.0
 *
 * @package    Quiz_Maker_Extra_Shortcodes
 * @subpackage Quiz_Maker_Extra_Shortcodes/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Quiz_Maker_Extra_Shortcodes
 * @subpackage Quiz_Maker_Extra_Shortcodes/includes
 * @author     Quiz Maker team <info@ays-pro.com>
 */
class Quiz_Maker_Extra_Shortcodes {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Quiz_Maker_Extra_Shortcodes_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'QUIZ_MAKER_EXTRA_SHORTCODES_VERSION' ) ) {
			$this->version = QUIZ_MAKER_EXTRA_SHORTCODES_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'quiz-maker-extra-shortcodes';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Quiz_Maker_Extra_Shortcodes_Loader. Orchestrates the hooks of the plugin.
	 * - Quiz_Maker_Extra_Shortcodes_i18n. Defines internationalization functionality.
	 * - Quiz_Maker_Extra_Shortcodes_Admin. Defines all hooks for the admin area.
	 * - Quiz_Maker_Extra_Shortcodes_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-extra-shortcodes-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-extra-shortcodes-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-quiz-maker-extra-shortcodes-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-quiz-maker-extra-shortcodes-public.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-user-progress-shortcode.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-user-history-quiz-shortcode.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-user-best-score-shortcode.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-user-activity-per-day-shortcode.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/partials/class-quiz-maker-quiz-activity-per-day-shortcode.php';

		$this->loader = new Quiz_Maker_Extra_Shortcodes_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Quiz_Maker_Extra_Shortcodes_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Quiz_Maker_Extra_Shortcodes_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Quiz_Maker_Extra_Shortcodes_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// Action hooks | Start
		
		// Quiz Maker Shorcodes / settings page
		$this->loader->add_action( 'ays_qm_settings_page_extra_shortcodes', $plugin_admin, 'ays_qm_settings_page_extra_shortcodes_content' );

		// Action hooks | End


		// ===== User Progress shortcode ====
		// User Progress shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_contents', $plugin_admin, 'ays_settings_page_user_progres_content', 1, 2 );

		// ===== User History Quiz shortcode ====
		// User History Quiz shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_contents', $plugin_admin, 'ays_settings_page_user_history_quiz_content', 3, 2 );
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_saves', $plugin_admin, 'ays_settings_page_user_history_quiz_save', 1, 2 );

		// ===== User Best Score shortcode ====
		// User Best Score shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_contents', $plugin_admin, 'ays_settings_page_user_best_score_content', 3, 2 );
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_saves', $plugin_admin, 'ays_settings_page_user_best_score_save', 1, 2 );

		// ===== User Activity Pper Day shortcode ====
		// User Activity Per Day shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_contents', $plugin_admin, 'ays_settings_page_user_activity_per_day_content', 4, 2 );

		// ===== Quiz Activity Pper Day shortcode ====
		// Quiz Activity Per Day shortcode / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_contents', $plugin_admin, 'ays_settings_page_quiz_activity_per_day_content', 5, 2 );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public 							= new Quiz_Maker_Extra_Shortcodes_Public( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_user_progress 			= new Quiz_Maker_Extra_Shortcodes_User_Progress_Public( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_user_history_quiz 		= new Quiz_Maker_Extra_Shortcodes_User_History_Quiz_Public( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_user_best_score 	    	= new Quiz_Maker_Extra_Shortcodes_User_Best_Score_Public( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_user_activity_per_day 	= new Quiz_Maker_Extra_Shortcodes_User_Activity_Per_Day_Public( $this->get_plugin_name(), $this->get_version() );
		$plugin_public_quiz_activity_per_day 	= new Quiz_Maker_Extra_Shortcodes_Quiz_Activity_Per_Day_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );


		$this->loader->add_action( 'wp_ajax_ays_extra_shortcode_user_reports_info_popup_ajax', $plugin_public_user_history_quiz, 'ays_extra_shortcode_user_reports_info_popup_ajax' );
		$this->loader->add_action( 'wp_ajax_nopriv_ays_extra_shortcode_user_reports_info_popup_ajax', $plugin_public_user_history_quiz, 'ays_extra_shortcode_user_reports_info_popup_ajax' );

        $this->loader->add_action( 'wp_ajax_ays_extra_shortcode_user_export_result_pdf', $plugin_public_user_history_quiz, 'ays_extra_shortcode_user_export_result_pdf' );
        $this->loader->add_action( 'wp_ajax_nopriv_ays_extra_shortcode_user_export_result_pdf', $plugin_public_user_history_quiz, 'ays_extra_shortcode_user_export_result_pdf' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Quiz_Maker_Extra_Shortcodes_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
