<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://https://ays-pro.com
 * @since      1.0.0
 *
 * @package    Quiz_Maker_Razorpay
 * @subpackage Quiz_Maker_Razorpay/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Quiz_Maker_Razorpay
 * @subpackage Quiz_Maker_Razorpay/public
 * @author     Quiz Maker team <info@ays-pro.com>
 */

use Razorpay\Api\Api;
use Razorpay\Api\Errors\SignatureVerificationError;

class Quiz_Maker_Razorpay_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Quiz_Maker_Razorpay_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Quiz_Maker_Razorpay_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/quiz-maker-razorpay-public.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Quiz_Maker_Razorpay_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Quiz_Maker_Razorpay_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

        wp_enqueue_script($this->plugin_name . '-razorpay', 'https://checkout.razorpay.com/v1/checkout.js');
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/quiz-maker-razorpay-public.js', array( 'jquery' ), $this->version, false );
	}

    public function calculateAmount( $amount, $currency ) {

        // Replace this constant with a calculation of the order's amount
        // Calculate the order total on the server to prevent
        // customers from directly manipulating the amount on the client

        $multiply_by = '1';
        $fixed = 2;
        if( $currency == 'jpy' ){
            $fixed = 0;
        }

        for( $i = 0; $i < $fixed; $i++ ){
            $multiply_by .= '0';
        }

        $amount = floatval( $amount ) * intval( $multiply_by );

        return $amount;
    }

    public function ays_quiz_razorpay_integration_functionality ($content, $quiz) {
        $id = isset($quiz['id']) && $quiz['id'] != '' ? intval($quiz['id']) : 0;
        $user_id = get_current_user_id();
        $options = (json_decode($quiz['options'], true) != null) ? json_decode($quiz['options'], true) : array();
    
        $actions = Quiz_Maker_Razorpay_Data::ays_get_setting('razorpay');
        $razorpay_res = ($actions === false) ? json_encode(array()) : $actions; 
        $razorpay = json_decode($razorpay_res, true);
    
        $razorpay_key_id = isset($razorpay['key_id']) && $razorpay['key_id'] != "" ? esc_attr($razorpay['key_id']) : '';
        $razorpay_key_secret = isset($razorpay['key_secret']) && $razorpay['key_secret'] != "" ? esc_attr($razorpay['key_secret']) : '';
        $razorpay_payment_terms = isset($razorpay['payment_terms']) && $razorpay['payment_terms'] != "" ? esc_attr($razorpay['payment_terms']) : '';
    
        $enable_razorpay = (isset($options['enable_razorpay']) && $options['enable_razorpay'] == 'on') ? true : false;
        $razorpay_amount = isset( $options['razorpay_amount'] ) ? floatval( $options['razorpay_amount'] ) : 0;
        $razorpay_currency = isset( $options['razorpay_currency'] ) && $options['razorpay_currency'] != '' ? strtoupper($options['razorpay_currency']) : '';
        $razorpay_amount = $this->calculateAmount( $razorpay_amount, $razorpay_currency );
        $razorpay_message = (isset($options['razorpay_message'])) ? stripslashes($options['razorpay_message']) : __('You need to pay to pass this quiz.', $this->plugin_name);

        $payment_type = (isset($options['payment_type']) && sanitize_text_field($options['payment_type']) != '') ? sanitize_text_field(esc_attr($options['payment_type'])) : 'prepay';
    
        $is_elementor_exists = Quiz_Maker_Data::ays_quiz_is_elementor();
    
        $connection = Quiz_Maker_Data::get_payment_connection('razorpay', $payment_type, $razorpay_payment_terms, $id, array());
    
        if ($enable_razorpay && $connection === true) {
            if ($razorpay_key_secret == '' || $razorpay_key_id == '') {
                $content = null;
            } else {
                $enqueue_scripts = true;
                if (!is_user_logged_in() && $razorpay_payment_terms == "lifetime") {
                    $enqueue_scripts = false;
                }
    
                if ($is_elementor_exists) {
                    $enqueue_scripts = false;
                }
                
                if ($razorpay_amount < 100) {
                    $enqueue_scripts = false;
                }

                $content = '';
                if ($enqueue_scripts) {
                    $order_id = '';
                    $check = true;
                    try {
                        require_once(QUIZ_MAKER_RAZORPAY_DIR . 'includes/razorpay-php/Razorpay.php');

                        $api = new Api($razorpay_key_id, $razorpay_key_secret);

                        $orderData = array(
                            'receipt'         => 'receipt_user_'.$user_id.'_quiz_'.$id,
                            'amount'          => $razorpay_amount,
                            'currency'        => $razorpay_currency
                        );
                        
                        $razorpayOrder = $api->order->create($orderData);
    
                        $order_id = $razorpayOrder->id;
                    } catch (Throwable $err) {
                        $check = false;
                    }

                    if ($check) {
                        if (!isset($razorpayOrder->error) && $order_id != '') {
                            $this->enqueue_scripts();
                            $this->enqueue_styles();
    
                            $quiz_razorpay_js_options = array(
                                'ajax_url' => admin_url('admin-ajax.php'),
                                "callbackUrl"=> QUIZ_MAKER_RAZORPAY_PUBLIC_URL .'/partials/quiz-maker-razorpay-callback.php',
                                'paymentTerms' => $razorpay_payment_terms,
                                'paymentType' => $payment_type,
                                'wrapClass' => '.ays_razorpay_div_' . $id,
                                'buttonId' => '#ays_quiz_razorpay_button_' . $id,
                                'quizId' => $id,
                                'razorpayOptions' => array(
                                    "key"=> $razorpay_key_id,
                                    "amount"=> $razorpay_amount,
                                    "currency"=> $razorpay_currency,
                                    "name"=> "Quiz Maker",
                                    "description"=> "Test Transaction",
                                    "order_id"=> $order_id,
                                ),
                            );
                            $content .= '
                                <script>
                                    if (typeof quizMakerRazorpay === "undefined") {
                                        var quizMakerRazorpay = [];
                                    }
                                    quizMakerRazorpay["' . $id . '"]  = "' . base64_encode(json_encode($quiz_razorpay_js_options)) . '";
                                </script>
                            ';
    
                            $quiz_razorpay_style = '';
                            $quiz_razorpay_type_js = 'prepay';
                            if ($payment_type == 'prepay') {
                                $quiz_razorpay_style = 'display:block;margin-bottom:40px';
                            } else {
                                $quiz_razorpay_type_js = 'postpay';
                                $quiz_razorpay_style = 'margin-bottom:40px';
                            }
                
                            $content .= '
                                <div class="ays_razorpay_wrap_div" style="' . $quiz_razorpay_style . '" data-type="' . $quiz_razorpay_type_js . '">
                                    <div class="ays_razorpay_details_div">
                                        ' . $razorpay_message . '
                                    </div>
                                    <div class="ays_razorpay_div_' . $id . '" style="display: none;">
                                        <button id="ays_quiz_razorpay_button_' . $id . '" class="ays_quiz_razorpay_submit" type="button">
                                            <div class="ays_quiz_razorpay_spinner ays_quiz_razorpay_hidden"></div>
                                            <span class="ays_quiz_razorpay_button_text">' . __("Pay now", $this->plugin_name) . '</span>
                                        </button>
                                        <span class="ays_quiz_razorpay_card_error" role="alert"></span>
                                    </div>
                                </div>';
                        } else {
                            $content = null;
                        }
                    } else {
                        $content = null;
                    }
                } else {
                    $content = null;
                }
            }
        } else {
            $content = null;
        }
    
        $razopay_payment_usermeta = Quiz_Maker_Data::get_quiz_payment_usermeta('razopay', get_current_user_id(), $id);
        $razopay_ontime_payment_usermeta = Quiz_Maker_Data::is_not_logged_in_user_paid($id, 'razopay', array());
    
        $quiz_razopay_payment_done = false;
    
        if (isset($razopay_payment_usermeta) && is_array($razopay_payment_usermeta) && !empty($razopay_payment_usermeta)) {
            $quiz_razopay_payment_done = (isset($razopay_payment_usermeta['purchased']) && $razopay_payment_usermeta['purchased'] != "") ? $razopay_payment_usermeta['purchased'] : false;
        }
    
        if ($quiz_razopay_payment_done) {
            $quiz_razopay_payment_done = true;
        }
    
        if ($enable_razorpay && $payment_type == "prepay") {
            if ($razorpay_payment_terms == 'onetime') {
                if ($razopay_ontime_payment_usermeta !== true) {
                    if ($content !== null) {
                        $content = null;
                    }
                }
            } else {
                if ($razopay_payment_usermeta !== false) {
                    if ($quiz_razopay_payment_done && $content !== null) {
                        $content = null;
                    }
                }
            }
        }
    
        return $content;
    }

}
