<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://ays-pro.com
 * @since      1.0.0
 *
 * @package    Quiz_Maker_Trainings
 * @subpackage Quiz_Maker_Trainings/public
 */

use Quiz_Maker\Cookie\Cookie;

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Quiz_Maker_Trainings
 * @subpackage Quiz_Maker_Trainings/public
 * @author     Quiz Maker team <info@ays-pro.com>
 */
class Quiz_Maker_Trainings_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Save training questions in the cookie
	 *
	 * @param $quiz_id
	 * @param $correct_questions
	 * @param $wrong_questions
	 *
	 * @return void
	 */
	public function save_training_questions( $quiz_id, $correct_questions, $wrong_questions ){
		$correct_cookie = new Cookie( QUIZ_MAKER_TRAININGS_COOKIE_KEY_CORRECT );
		$wrong_cookie = new Cookie( QUIZ_MAKER_TRAININGS_COOKIE_KEY_WRONG );

		$wrong_cookie_value = array(
			$quiz_id => $wrong_questions
		);
		if ( Cookie::exists( QUIZ_MAKER_TRAININGS_COOKIE_KEY_WRONG ) ) {
			$cookie_value = json_decode( base64_decode( Cookie::get( QUIZ_MAKER_TRAININGS_COOKIE_KEY_WRONG ) ), true );
			if( isset( $cookie_value[$quiz_id] ) ) {
				$cookie_value[$quiz_id] = array_diff( $cookie_value[$quiz_id], $correct_questions );
				$wrong_cookie_value[$quiz_id] = array_unique( array_merge( $wrong_questions, $cookie_value[$quiz_id] ) );
			}
		}

		$wrong_cookie->setValue( base64_encode( json_encode( $wrong_cookie_value ) ) );
		$wrong_cookie->setMaxAge(60 * 60 * 24 * 30);
		$wrong_cookie->save();

		$correct_cookie_value = array(
			$quiz_id => $correct_questions
		);
		if ( Cookie::exists( QUIZ_MAKER_TRAININGS_COOKIE_KEY_CORRECT ) ) {
			$cookie_value = json_decode( base64_decode( Cookie::get( QUIZ_MAKER_TRAININGS_COOKIE_KEY_CORRECT ) ), true );
			if( isset( $cookie_value[$quiz_id] ) ) {
				$cookie_value[$quiz_id] = array_diff( $cookie_value[$quiz_id], $wrong_questions );
				$correct_cookie_value[$quiz_id] = array_unique( array_merge( $correct_questions, $cookie_value[$quiz_id] ) );
			}
		}

		$correct_cookie->setValue( base64_encode( json_encode( $correct_cookie_value ) ) );
		$correct_cookie->setMaxAge(60 * 60 * 24 * 30);
		$correct_cookie->save();
	}

	/**
	 * Get wrong answered questions form cookie
	 *
	 * @param $questions
	 * @param $quiz_id
	 *
	 * @return mixed
	 */
	public function get_training_wrong_questions( $questions, $quiz_id ){
		$cookie_value = Cookie::get( QUIZ_MAKER_TRAININGS_COOKIE_KEY_WRONG );

		if( isset( $_GET['reset_quiz'] ) ){
			$this->delete_training_wrong_questions();
			$this->redirect_to_restart();
		}

		if( $cookie_value ) {
			$saved_questions = json_decode( base64_decode( $cookie_value ), true );
			if( !empty( $saved_questions ) ) {
				if( isset( $saved_questions[$quiz_id] ) && ! empty( $saved_questions[$quiz_id] ) ){
					$questions = $saved_questions[$quiz_id];
				}
			}
		}

		return $questions;
	}

	/**
	 * Get correct answered questions form cookie
	 *
	 * @param $questions
	 * @param $quiz_id
	 *
	 * @return mixed
	 */
	public function get_training_correct_questions( $questions, $quiz_id ){
		$cookie_value = Cookie::get( QUIZ_MAKER_TRAININGS_COOKIE_KEY_CORRECT );

		if( isset( $_GET['reset_quiz'] ) ){
			$this->delete_training_correct_questions();
			$this->redirect_to_restart();
		}

		if( $cookie_value ) {
			$saved_questions = json_decode( base64_decode( $cookie_value ), true );
			if( !empty( $saved_questions ) ) {
				if( isset( $saved_questions[$quiz_id] ) && ! empty( $saved_questions[$quiz_id] ) ){
					$questions = $saved_questions[$quiz_id];
				}
			}
		}

		return $questions;
	}

	/**
	 * Delete correct answered questions form cookie
	 *
	 * @return void
	 */
	public function delete_training_correct_questions(){
		$cookie_value = new Cookie( QUIZ_MAKER_TRAININGS_COOKIE_KEY_CORRECT );
		$cookie_value->delete();
	}

	/**
	 * Delete wrong answered questions form cookie
	 *
	 * @return void
	 */
	public function delete_training_wrong_questions(){
		$cookie_value = new Cookie( QUIZ_MAKER_TRAININGS_COOKIE_KEY_WRONG );
		$cookie_value->delete();
	}

	/**
	 * Redirect to the same page after deleting cookies.
	 *
	 * @return void
	 */
	public function redirect_to_restart(){
		$link = "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
		if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === "on"){
			$link = "https" . $link;
		}else{
			$link = "http" . $link;
		}

		$link = remove_query_arg( array( 'reset_quiz' ), $link );
		wp_redirect( $link );
	}

	/**
	 * Get restart button content
	 *
	 * @param $quiz_id
	 *
	 * @return mixed
	 */
	public function get_restart_training_button( $content, $quiz_id ){
		$content = "";
		ob_start();
	    ?>

		<div class="ays-quiz-training-main-container">
			<div class="ays-quiz-training-container" style="text-align:center;">
				<button type='button' class='action-button ays_restart_training_button'>
					<span><?php echo __("Reset quiz", $this->plugin_name) ?></span>
				</button>
			</div>
		</div>

		<?php
        $content = ob_get_clean();

		return $content;
	}
}
