<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://ays-pro.com
 * @since      1.0.0
 *
 * @package    Quiz_Maker_User_Dashboard
 * @subpackage Quiz_Maker_User_Dashboard/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Quiz_Maker_User_Dashboard
 * @subpackage Quiz_Maker_User_Dashboard/includes
 * @author     Quiz Maker team <info@ays-pro.com>
 */
class Quiz_Maker_User_Dashboard {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Quiz_Maker_User_Dashboard_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'QUIZ_MAKER_USER_DASHBOARD_VERSION' ) ) {
			$this->version = QUIZ_MAKER_USER_DASHBOARD_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'quiz-maker-user-dashboard';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Quiz_Maker_User_Dashboard_Loader. Orchestrates the hooks of the plugin.
	 * - Quiz_Maker_User_Dashboard_i18n. Defines internationalization functionality.
	 * - Quiz_Maker_User_Dashboard_Admin. Defines all hooks for the admin area.
	 * - Quiz_Maker_User_Dashboard_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-user-dashboard-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-user-dashboard-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-quiz-maker-user-dashboard-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-quiz-maker-user-dashboard-public.php';

		//User Dashboard Activity Per Day
		require_once plugin_dir_path( dirname( __FILE__ ) ). 'public/partials/class-quiz-maker-user-advanced-activity-per-day.php';
		
		//User Dashboard Results 
		require_once plugin_dir_path( dirname( __FILE__ ) ). 'public/partials/class-quiz-maker-user-advanced-results.php';

		//User Dashboard Results By Quiz
		require_once plugin_dir_path( dirname( __FILE__ ) ). 'public/partials/class-quiz-maker-user-advanced-user-page.php';

		//User Dashboard Best Score By Quiz
		require_once plugin_dir_path( dirname( __FILE__ ) ). 'public/partials/class-quiz-maker-user-advanced-best-score.php';

		//User Dashboard User Progress By Quiz
		require_once plugin_dir_path( dirname( __FILE__ ) ). 'public/partials/class-quiz-maker-user-advanced-user-progress.php';
		
		$this->loader = new Quiz_Maker_User_Dashboard_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Quiz_Maker_User_Dashboard_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Quiz_Maker_User_Dashboard_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Quiz_Maker_User_Dashboard_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// Add menu item
		$this->loader->add_action( 'ays_qm_advanced_user_dashboard', $plugin_admin, 'ays_qm_settings_page_advanced_user_dashboard' );

		//Action hook
		//Advanced user dashboard
		$this->loader->add_filter( 'ays_qm_settings_page_advanced_user_dashboard_contents', $plugin_admin, 'ays_settings_page_advanced_user_dashboard', 3, 2 );
		$this->loader->add_filter( 'ays_qm_settings_page_extra_shortcodes_saves', $plugin_admin, 'ays_settings_page_user_dashboard_quiz_save', 1, 2 );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Quiz_Maker_User_Dashboard_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		// AJAX Results By Quiz 
		$this->loader->add_action( 'wp_ajax_get_results_by_quiz', $plugin_public, 'get_results_by_quiz' );
        $this->loader->add_action( 'wp_ajax_nopriv_get_results_by_quiz', $plugin_public, 'get_results_by_quiz' );

		// AJAX Details
		$this->loader->add_action( 'wp_ajax_get_user_reports_info_popup_ajax', $plugin_public, 'get_user_reports_info_popup_ajax' );
		$this->loader->add_action( 'wp_ajax_nopriv_get_user_reports_info_popup_ajax', $plugin_public, 'get_user_reports_info_popup_ajax' );

		// AJAX Export User Results PDF
		$this->loader->add_action( 'wp_ajax_get_user_reports_pdf_ajax', $plugin_public, 'get_user_reports_pdf_ajax' );
		$this->loader->add_action( 'wp_ajax_nopriv_get_user_reports_pdf_ajax', $plugin_public, 'get_user_reports_pdf_ajax' );

		// AJAX User Best Score by quiz
		$this->loader->add_action( 'wp_ajax_user_best_score_by_quiz_ajax', $plugin_public, 'user_best_score_by_quiz_ajax' );
		$this->loader->add_action( 'wp_ajax_nopriv_user_best_score_by_quiz_ajax', $plugin_public, 'user_best_score_by_quiz_ajax' );

		// AJAX User Progress by quiz
		$this->loader->add_action( 'wp_ajax_user_progress_by_quiz_ajax', $plugin_public, 'user_progress_by_quiz_ajax' );
		$this->loader->add_action( 'wp_ajax_nopriv_user_progress_by_quiz_ajax', $plugin_public, 'user_progress_by_quiz_ajax' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Quiz_Maker_User_Dashboard_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
