<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://ays-pro.com
 * @since      1.0.0
 *
 * @package    Quiz_Maker_Woocommerce
 * @subpackage Quiz_Maker_Woocommerce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Quiz_Maker_Woocommerce
 * @subpackage Quiz_Maker_Woocommerce/includes
 * @author     Quiz Maker team <info@ays-pro.com>
 */
class Quiz_Maker_Woocommerce {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Quiz_Maker_Woocommerce_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'QUIZ_MAKER_WOOCOMMERCE_VERSION' ) ) {
			$this->version = QUIZ_MAKER_WOOCOMMERCE_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'quiz-maker-woocommerce';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Quiz_Maker_Woocommerce_Loader. Orchestrates the hooks of the plugin.
	 * - Quiz_Maker_Woocommerce_i18n. Defines internationalization functionality.
	 * - Quiz_Maker_Woocommerce_Admin. Defines all hooks for the admin area.
	 * - Quiz_Maker_Woocommerce_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-woocommerce-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-quiz-maker-woocommerce-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-quiz-maker-woocommerce-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-quiz-maker-woocommerce-public.php';

		$this->loader = new Quiz_Maker_Woocommerce_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Quiz_Maker_Woocommerce_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Quiz_Maker_Woocommerce_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Quiz_Maker_Woocommerce_Admin( $this->get_plugin_name(), $this->get_version() );

		// Quiz Maker Integrations / Woocommerce / settings page
		$this->loader->add_filter( 'ays_qm_settings_page_integrations_contents', $plugin_admin, 'ays_woocommerce_settings_page_content', 2, 2 );

		// Quiz Maker Integrations / Woocommerce / quiz page
		$this->loader->add_filter( 'ays_qm_quiz_page_integrations_contents', $plugin_admin, 'ays_quiz_page_woocommerce_content', 2, 2 );
		$this->loader->add_filter( 'ays_qm_quiz_page_integrations_options', $plugin_admin, 'ays_quiz_page_woocommerce_options', 1, 2 );
		$this->loader->add_filter( 'ays_qm_quiz_page_integrations_saves', $plugin_admin, 'ays_quiz_page_woocommerce_save', 1, 2 );

		// Mad Mimi integration / front-end
		// $this->loader->add_filter( 'ays_qm_woocommerce_front_end_integrations_options', $plugin_admin, 'ays_front_end_mad_mimi_options', 1, 2 );
		$this->loader->add_filter( 'ays_qm_woocommerce_front_end_integrations', $plugin_admin, 'ays_front_end_woocommerce_functional', 1, 1 );

		$this->loader->add_action( 'wp_ajax_ays_quiz_woocommerce_get_woocommerce_products', $plugin_admin, 'ays_quiz_woocommerce_get_woocommerce_products' );
        $this->loader->add_action( 'wp_ajax_nopriv_ays_quiz_woocommerce_get_woocommerce_products', $plugin_admin, 'ays_quiz_woocommerce_get_woocommerce_products' );

        $this->loader->add_action( 'wp_ajax_ays_quiz_woocommerce_create_item', $plugin_admin, 'ays_quiz_woocommerce_create_item' );
        $this->loader->add_action( 'wp_ajax_nopriv_ays_quiz_woocommerce_create_item', $plugin_admin, 'ays_quiz_woocommerce_create_item' );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Quiz_Maker_Woocommerce_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Quiz_Maker_Woocommerce_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
